/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb.ejb3;

import java.util.Hashtable;

import javax.naming.NamingException;

import org.ow2.bonita.facade.APIAccessor;
import org.ow2.bonita.facade.CommandAPI;
import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.interceptor.EJBCommandAPIInterceptor;
import org.ow2.bonita.util.AccessorProxyUtil;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.Misc;


/**
 * @author Guillaume Porcher
 *
 */
public class EJB3APIAccessorImpl extends EJB3QueryAPIAccessorImpl implements APIAccessor {

  public EJB3APIAccessorImpl(Hashtable<String, String> jndiEnvironment) {
    super(jndiEnvironment);
  }
  
  public CommandAPI getCommandAPI() {
    CommandAPI ejbCommandAPI;
    try {
      ejbCommandAPI = AccessorProxyUtil.getClientAPI(CommandAPI.class,
          Misc.lookup(AccessorUtil.COMMANDAPI_JNDINAME, jndiEnvironment)
      );
    } catch (final NamingException e) {
      throw new BonitaRuntimeException(e);
    }
    return new EJBCommandAPIInterceptor(ejbCommandAPI);

  }
  
  public ManagementAPI getManagementAPI() {
    try {
      return AccessorProxyUtil.getClientAPI(ManagementAPI.class,
          Misc.lookup(AccessorUtil.MANAGEMENT_JNDINAME, jndiEnvironment));
    } catch (final NamingException e) {
      throw new BonitaRuntimeException(e);
    }
  }

  public RuntimeAPI getRuntimeAPI() {
    try {
      return AccessorProxyUtil.getClientAPI(RuntimeAPI.class,
          Misc.lookup(AccessorUtil.RUNTIMEAPI_JNDINAME, jndiEnvironment));
    } catch (final NamingException e) {
      throw new BonitaRuntimeException(e);
    }
  }

}
