/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.interceptor;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.rmi.RemoteException;

import org.ow2.bonita.facade.exception.BonitaInternalException;

// This class is only there to wrap try{...} catch(RemoteException e){...}
public class ClientAPIInterceptor implements InvocationHandler {

  private final Object api;
  private final String queryList;

  public ClientAPIInterceptor(final Object api) {
    this.api = api;
    this.queryList = null;
  }

  public ClientAPIInterceptor(final Object api, final String queryList) {
    this.api = api;
    this.queryList = queryList;
  }

  public Object invoke(final Object proxy, final Method method, final Object[] args) throws Throwable {
    try {
      Class< ? >[] params;
      Object[] methodsArgs;
      if (this.queryList != null) {
        final Class< ? >[] p = method.getParameterTypes();
        if (p != null) {
          params = new Class< ? >[p.length + 1];
          for (int i = 0; i < p.length; i++) {
            params[i] = p[i];
          }
          params[p.length] = String.class;
        } else {
          params = new Class< ? >[] {String.class};
        }
        if (args != null) {
          methodsArgs = new Object[args.length + 1];
          for (int i = 0; i < args.length; i++) {
            methodsArgs[i] = args[i];
          }
          methodsArgs[args.length] = this.queryList;
        } else {
          methodsArgs = new Object[] {this.queryList};
        }
      } else {
        params = method.getParameterTypes();
        methodsArgs = args;
      }
      final Method m = this.api.getClass().getMethod(method.getName(), params);
      return m.invoke(this.api, methodsArgs);
    } catch (final Exception e) {
      if (e instanceof InvocationTargetException) {
        final Throwable invocationExceptionCause = e.getCause();
        if (invocationExceptionCause instanceof RemoteException) {
          final RemoteException remoteException = (RemoteException) invocationExceptionCause;
          final Throwable remoteCause = getRemoteCause(remoteException);
          InterceptorUtil.manageInvokeExceptionCause(method, remoteCause);
        } else {
          throw invocationExceptionCause;
        }
      }
      throw new BonitaInternalException("Ouch! Unexpected error: " + e, e);
    }
  }

  private Throwable getRemoteCause(final RemoteException e) {
    Throwable t = e;
    while (t instanceof RemoteException) {
      t = t.getCause();
    }
    return t;
  }

}

