package org.ow2.bonita.facade.interceptor;

import java.lang.reflect.Method;

import org.ow2.bonita.util.BonitaException;

/**
 * Util class used by the engine to access EJB2 API Accessor and manage invocation exception
 * @author Thomas Gueze
 *
 */
final class InterceptorUtil {

  private InterceptorUtil() { }

  /**
   * Throw the throwable passed if it's an RuntimeException or if it's an exception declared by the method
   * If not, throw nothing and return
   */
  @SuppressWarnings("unchecked")
  static void manageInvokeExceptionCause(final Method m, final Throwable t) throws BonitaException {
    if (t instanceof RuntimeException) {
      throw (RuntimeException) t;
    }
    final Class[] throwedClasses = m.getExceptionTypes();
    for (final Class clazz : throwedClasses) {
      throwEx(clazz, t);
    }
  }

  @SuppressWarnings("unchecked")
  private static <T extends BonitaException> void throwEx(final Class<T> clazz, final Throwable t) throws T {
    if (t != null && clazz.isAssignableFrom(t.getClass())) {
      throw (T) t;
    }
  }

}
